//
//  Function Integrand:  Helper function for StochMon.  Reads
//  FSpectra "gammaValue" "psd_1" and "psd_2" and copies
//  gammaValue(f)/[f^{6}*psd_1(f)*psd_2(f)] into another  FSpectrum "out".  
//   version: n/a ($Date: 2005-09-16 14:30:25 -0700 (Fri, 16 Sep 2005) $;
//                 $Id: Integrand.cc 3921 2005-09-16 21:30:25Z john.whelan $)
//   authors: Marc J. Cenac (mjcenac@loyno.edu)
//            Brian O'Reilly (irish@ligo-la.caltech.edu)
//            John T. Whelan (jtwhelan@loyno.edu)
//original by: Kevin C. Schlaufman (kcs149@psu.edu)
//             Patrick J. Sutton (psutton@ligo.caltech.edu)
//
///////////////////////////////////////////////////////////////////////////

#include "Integrand.hh"
#include <cmath>
#include "LLOLHO_Overlap.hh"
using namespace std;

void integrand(FSpectrum& out, FSpectrum& psd_1, FSpectrum& psd_2, double secPerStride, FSpectrum& gammaValue)
{
//----- Set FSpectrum out equal to FSpectrum psd.
  out = psd_1; //--we should really check to make sure that they are the same but we don't
 
  //----- Initialize items necessary to prepare the integrand.
  int i;
  int nsteps = out.getNStep();
  double low_freq = out.getLowFreq();
  double f_step = out.getFStep();
  float *data = new float[nsteps + 1];
  float *data_2 = new float[nsteps + 1];
  float *gamma = new float[nsteps + 1];
  double freq = low_freq + (i * f_step);
  // The following prefactor is in units of (attostrain)^{-4} s^{-3}
  const double myprefactor = 1.1300758118;
  //----- Copy data from out object to data array.
  out.getData(nsteps + 1, data);
  psd_2.getData(nsteps + 1, data_2);
  gammaValue.getData(nsteps + 1, gamma);
 //----- Loop to invert PSDed data .  Also divide by f^(6).
  
  for (int i = 0; i < (nsteps + 1); i++)
    {
            
      data[i] = 1./(data[i]); //--put array in the denominator.

      data[i] *= 1./(data_2[i]); 

      freq = low_freq + (i * f_step); 
      //      std::cerr << " data[" << i << "] is " << data[i] << "    data_2[" << i << "] is " << data_2[i] << "   freq:" << freq << "\n";
      data[i] *= pow(freq,(-6)); 
      data[i] *= ((myprefactor * secPerStride)/60);
      data[i] *= (gamma[i] * gamma[i]); //--multiply data by [gamma(f)]^2 here
      
    }
  
  //----- Send data back to FSpectrum object.
  out.setData(nsteps + 1, data);
  
  
  //Release the memory allocate by new
    delete[] data;  
    delete[] data_2;
    delete[] gamma; 
}
